import matplotlib.pyplot as plt, matplotlib as mpl
import seaborn as sns
from matplotlib.ticker import FuncFormatter
import numpy as np
from pathlib import Path

figpath = Path( __file__ ).parents[0]
#### MATPLOTLIB CONFIGS #####
font = {'family' : 'sans-serif',
        'weight' : 'light',
        'size'   : 9 }

mpl.rc( 'font', **font )

plt.rcParams[ 'lines.linewidth' ] = 0.7

plt.rcParams[ 'xtick.direction' ] = 'in'
plt.rcParams[ 'xtick.major.size' ] = 2.0
plt.rcParams[ 'xtick.major.pad' ] = 2.5

plt.rcParams[ 'ytick.direction' ] = 'in'
plt.rcParams[ 'ytick.major.size' ] = 2.0
plt.rcParams[ 'ytick.major.pad' ] = 2.5

### SHARED PLOTTING TOOLS ###
markersize_s21 = 0.5
sptheory_color = '#1989C8'

# colormap for 2d plots
# cmap2d = sns.light_palette("#005FD9", reverse=True, as_cmap=True)
# cmap2d_r = sns.light_palette("#005FD9", reverse=False, as_cmap=True)
cmap2d = sns.color_palette( "crest", as_cmap=True )
cmap2d_r = sns.color_palette( "crest_r", as_cmap=True )
def my_formatter(x, pos):
    """Format 1 as 1, 0 as 0, and all values whose absolute values is between
    0 and 1 without the leading "0." (e.g., 0.7 is formatted as .7 and -0.4 is
    formatted as -.4)."""
    val_str = '{:g}'.format(x)
    if np.abs(x) == 0.0:
        return val_str.replace("0.0", "0", 1)
    else:
        return val_str
major_formatter = FuncFormatter(my_formatter)

def multiple_formatter(denominator=2, number=np.pi, latex='\pi'):
#  function for automatically setting ticks based on number, denominator, 
#  and a LaTeX symbol. Defaults to 2\pi.  
        def gcd(a, b):
            while b:
                a, b = b, a%b
            return a
        def _multiple_formatter(x, pos):
            den = denominator
            num = np.int(np.rint(den*x/number))
            com = gcd(num,den)
            (num,den) = (int(num/com),int(den/com))
            if den==1:
                if num==0:
                    return r'$0$'
                if num==1:
                    return r'$%s$'%latex
                elif num==-1:
                    return r'$-%s$'%latex
                else:
                    return r'$%s%s$'%(num,latex)
            else:
                if num==1:
                    return r'$\frac{%s}{%s}$'%(latex,den)
                elif num==-1:
                    return r'$\frac{-%s}{%s}$'%(latex,den)
                else:
                    return r'$\frac{%s%s}{%s}$'%(num,latex,den)
        return _multiple_formatter

class Multiple:
        def __init__(self, denominator=2, number=np.pi, latex='\pi'):
            self.denominator = denominator
            self.number = number
            self.latex = latex

        def locator(self):
            return plt.MultipleLocator(self.number / self.denominator)
        def formatter(self):
            return plt.FuncFormatter(multiple_formatter(self.denominator, self.number, self.latex))

def bstar(ax, Bstar=0.33481856, dir='in', padd=0, size=9) -> None:
    """ Add B* to an x-axis
    """
    secax = ax.twiny()
    secax.xaxis.set_ticks_position("bottom")
    secax.set_xlim(ax.get_xlim())
    secax.set_xticks([Bstar])
    secax.tick_params(axis='x', colors='purple')
    secax.tick_params(direction=dir, labelsize = size, pad = padd)
    secax.set_xticklabels(['$B^*$'])