#%%
%matplotlib qt
import numpy as np, json
import matplotlib.pyplot as plt

def _load_file( f ) :
    ''' load file and convert to nparrays
    returns: dictionary with data in it
    '''
    with open( f ) as json_file :
        data_str = json.load( json_file )
    data = dict()
    for k in data_str : # convert strings to ints
        data[ float( k ) ] = data_str[ k ]

    for sweep in data.values() : # convert to np arrays
        for k, l in sweep.items() :
            sweep[ k ] = np.array( l )
    return data

def _get_min_f( data ) :
    ''' return minimum frequency in an angle sweep
    '''
    get_smallest_f = lambda sweep : min( sweep['f'] )
    smallest_f = min( map( get_smallest_f, data.values() ) )
    return smallest_f

def plot_angle_experiment( smallest_f=None ):
    ''' load and plot angle sweep
    smallest_f: smallest frequency to plot
    '''
    f = 'angle_experiment.json'
    data = _load_file( f )
    if smallest_f is None:
        smallest_f = _get_min_f( data )/1e9

    fig = plt.figure()
    ax = fig.add_axes( [0.05, 0.1, 0.9, 0.8],
                projection='polar' )

    for b, sweeps in data.items() :
        ax.errorbar( sweeps['angle']*np.pi/180, sweeps['f']/1e9, 
                yerr= sweeps['f_err' ]/1e9, linestyle='',
                linewidth=0.5, marker='o', 
                markersize=1, label=f'$|B|$ = {b}T' )

    ax.set_rlim( smallest_f )
    ax.set_yticklabels( [] )
    ax.legend( fontsize=11, loc='upper left', 
                bbox_to_anchor=(0,1),
                bbox_transform=fig.transFigure )

def plot_angle_theory( smallest_f=None ):
    ''' load and plot angle sweep
    smallest_f: smallest frequency to plot
    '''
    f = 'angle_theory.json'
    data = _load_file( f )
    if smallest_f is None:
        smallest_f = _get_min_f( data )

    smallest_f = _get_min_f( data )
    fig = plt.figure()
    ax = fig.add_axes( [0.05, 0.1, 0.9, 0.8],
                projection='polar' )

    for b, sweeps in data.items() :
        ax.errorbar( sweeps['angle']*np.pi/180, sweeps['f'], 
                linewidth=0.75, label=f'$|B|$ = {b}T' )

    ax.set_rlim( smallest_f )
    ax.set_yticklabels( [] )
    ax.legend( fontsize=11, loc='upper left', 
                bbox_to_anchor=(0,1),
                bbox_transform=fig.transFigure )

plot_angle_experiment( smallest_f=4.8 )
plot_angle_theory( smallest_f=4.8 )
# %%
