#%% IMPORTS
# import matplotlib.pyplot as plt, matplotlib as mpl
import matplotlib.pyplot as plt, matplotlib as mpl
%matplotlib qt
import numpy as np, json
from glob import glob
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from figures import bstar, major_formatter, cmap2d, sptheory_color
# import figures # set figure defaults

def plot_fb( ax, f) :
    ''' plot f(b,t) with theory, and inset
    ax: axes to draw plot on
    f: file to plot
    '''

    with open( f ) as json_file:
        data = json.load(json_file)
    
    for k, d in data.items() :
        data[k] = np.array( d )
    ax.axis('on')
    ebar_settings = { 'elinewidth' : 0.5, 
                    'markeredgewidth' : 0.5 }
    ax.errorbar( data['b_exp'], data['f_exp']/1e9, yerr = data['df_exp']/1e9, color = 'k', 
                marker='.', 
                markersize=2.5,
                linestyle='', zorder=3, **ebar_settings )

    stheory, = ax.plot( data['b_theory'], data['f_stheory']/1e9,  marker=' ', 
            color = 'black', linewidth=0.7,
            linestyle='-', zorder=2 )
    sptheory, = ax.plot( data['b_theory'], data['f_sptheory']/1e9,  marker=' ', 
            color = sptheory_color,
            linestyle='-', zorder=1, linewidth=1 )
    ax.set_ylim( 5.05, 5.63 )
    ax.set_ylabel('$f_r$ (GHz)', labelpad=-9, y=0.47 )
    ax.set_yticks( (5.1, 5.6) )
    ax.set_xlabel( '$B_x$ (T)', labelpad=-7.5, x=0.33 )
    ax.set_xticks( (0, 0.5 ) )
    stheory.set_label('S')
    sptheory.set_label('P')
    # leg = ax.legend(bbox_to_anchor=(0,1.02,1,0.2), loc="lower left",
    #             mode="expand", borderaxespad=0, ncol=2, fontsize=10, 
    #             frameon=True, borderpad=0.1 )

    ########### INSET ############ 
    axins = inset_axes(ax, width="100%", height="100%",loc='lower left',
                   bbox_to_anchor=(0.03,0.08,0.52,0.52), 
                   bbox_transform=ax.transAxes) # loc=3,borderpad=1.2) 
    plot_inset( data, axins )

def plot_inset( data, axins ) :
    """ Plot nb( T ) inset
    """
    ebar_settings = { 'elinewidth' : 0.5,
                    'markeredgewidth' : 0.5 }
    axins.errorbar( data['b_exp'], data['np_exp'], color = 'black', marker='.', 
            markersize=2, yerr=data['dnp_exp'],
            linestyle='', zorder=3, **ebar_settings )
    axins.plot( data['b_theory'], data['np_theory'], color=sptheory_color, linewidth=0.7 )
    # axins.errorbar( data['b_exp'], (data['f_sptheory'][::2]-data['f_exp'])/1e9, 
    #             yerr = 0.025*data['f_sptheory'][::2]/1e9, color = 'b', marker=' ', 
    #             markersize=0.1, 
    #             linestyle='', zorder=3, label = '$SP$' )
    fsize_ins = 9
    axins.set_ylim( (-0.29, 1.3) )
    axins.set_yticks( (0.0, 1.0) )
    axins.set_yticklabels( ( 0, 1 ), size=fsize_ins )
    axins.set_ylabel('$n_\mathrm{Pauli}$', size=fsize_ins, 
            labelpad = 2.5, rotation=-90, y = 0.48 )
    axins.tick_params(axis='both', pad=1)
    axins.xaxis.set_major_formatter(major_formatter)

    axins.yaxis.set_label_position("right")
    axins.yaxis.tick_right()

    axins.xaxis.set_visible(True)
    axins.set_xticks( (0,0.5) )
    axins.set_xticklabels( (0,0.5), size=fsize_ins )
    axins.set_xlabel('$B_x$ (T)', labelpad=-7.5, x=0.3 )
    bstar( axins, padd=2 )

def plot_cmap(ax, result, bs, ts, vmin=None, vmax=None, **kwargs ) :
    ''' plot result, b, t
    return ax, cbar

    '''
    X, Y = np.meshgrid( ts, bs )
    # cmap = sns.color_palette("crest", as_cmap=True)
    
    # cmap = sns.dark_palette("crest_r", reverse=True, as_cmap=True)
    pcm = ax.pcolormesh( Y, X, result, 
            cmap=cmap2d, vmin=vmin, vmax=vmax, **kwargs )
    pcm.set_edgecolor('face')
    ax.set_xlabel( '$B_x$ (T)', labelpad=-9, x=0.33 )
    ax.set_xticks( ( 0, 0.5 ) )
    ax.set_ylabel( '$T$ (K)', labelpad=-10, y=0.48 )
    ax.set_yticks( (0.1, 0.5, 1.0) )
    ax.set_yticklabels( (0.1, None, 1.0) )

    ax.xaxis.set_major_formatter(major_formatter)
    ax.tick_params(axis='both', which='major', pad=1)
    ax.tick_params( direction='out' )

    return pcm, cmap2d

def plot_nmap( ax, f, **kwargs ) :
    ''' plot file
    f: file to plot
    kwargs: passed along to plot_cmap
    '''
    with open( f ) as json_file:
        data = json.load(json_file)
    pcm, cmap = plot_cmap(ax, data['np'], data['bs'], data['ts'], **kwargs )
    return pcm, cmap

def make_cbar( ax, pcm ) :
    ''' make colorbar for diff maps '''
    
    cax = inset_axes(ax,
            width="100%",  # width = 50% of parent_bbox width
            height="3%",  # height : 5%
            loc='upper left',
            bbox_to_anchor=(0, 0, 1, 1.05),
            bbox_transform = ax.transAxes,
            borderpad=0 )

    cbar = ax.figure.colorbar( pcm, ax=ax, cax=cax,
                        orientation='horizontal', ticks=(0, 1.0) )

    cbar.ax.set_ylabel( '$n_p$', size=9, rotation=0, labelpad=-40, y=1.5 )
    cbar.ax.tick_params( labelsize=9, direction='out', pad=0.0 )
    cbar.ax.set_xticklabels( ( 0, 1 ) )
    cbar.ax.xaxis.set_label_position('top')
    cbar.ax.xaxis.set_ticks_position('top')

# DRAW THE FIGURE
# fig, ax = plt.subplots( 2, 2, figsize=(3.5,3.3) )
fig = plt.figure( figsize=(3.5,3.5) )
gs = fig.add_gridspec( ncols=2, nrows=7 )
fig.add_subplot(gs[0:4,:])
fig.add_subplot(gs[4:, 0])
fig.add_subplot(gs[4:, 1])
ax = fig.axes
# for a in ax[1:] : a.set_aspect( 'equal' )

# ### FIG 2a ####
plot_fb(ax[0],'f_bz_pauli.json')
# ax[0][0].set_aspect('equal')
ax[0].xaxis.set_major_formatter(major_formatter)
bstar(ax[0], padd = 2.5)

#### FIG 2c/d ####
# ### c ####
crange = { 'vmin' : 0, 'vmax' : 1.0 }
pcm, cmap = plot_nmap(ax[1], 'np_bt_exp2d_pauli.json', **crange ) 
make_cbar( ax[1], pcm )
bstar(ax[1], dir='out', padd=2)
ax[1].text(0.015, 0.12, 'EXP.', 
    horizontalalignment='left',
    color='white',
    fontsize=9,
    verticalalignment='top', transform=ax[1].transAxes)

# ### d ####
pcm, cmap = plot_nmap(ax[2], 'np_bt_theory2d_pauli.json', **crange )
make_cbar( ax[2], pcm )
bstar(ax[2], dir='out', padd=2)
ax[2].text(0.015, 0.24, 'PAULI\nTHEORY', 
    horizontalalignment='left',
    color='white',
    fontsize=9,
    verticalalignment='top', transform=ax[2].transAxes)


gs.tight_layout( fig, pad=1, w_pad=0.0, h_pad=1.0 )
# fig.tight_layout() # pad=0.5, w_pad=0.0, h_pad=0.0)

#### ALIGNED LABELS FOR SUBPLOTS ########
coords = { 'a' : ( 0.00, 0.97 ),
            'b' : ( 0.00, 0.45 ),
            'c' : ( 0.495, 0.45 ),
        }
for lbl, xy in coords.items() :
    fig.text( *xy, lbl, 
            fontweight = 'bold',
            fontsize = 10 ) 

#%%
fig.savefig( "fig_pauli_bt.pdf" )
