#%% plot bsweeps for s58 sup fig
import matplotlib.pyplot as plt, numpy as np, os, json
from pathlib import Path
import seaborn as sns
from figures import figpath

from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.colors import LinearSegmentedColormap
from matplotlib.ticker import FuncFormatter

export_dir = figpath / 'supfigs'

def my_formatter(x, pos):
    """Format 1 as 1, 0 as 0, and all values whose absolute values is between
    0 and 1 without the leading "0." (e.g., 0.7 is formatted as .7 and -0.4 is
    formatted as -.4)."""
    val_str = '{:g}'.format(x)
    if np.abs(x) == 0.0:
        return val_str.replace("0.0", "0", 1)
    else:
        return val_str
major_formatter = FuncFormatter(my_formatter)

def plot_f(ax=None, fname='f_by57.json', label = " ", color='black'):
    """plot f_by57.json"""
    with open( f'{export_dir}/'+fname ) as infile:
        f = json.load( infile )
    
    if ax is None: 
        fig, ax = plt.subplots()
    
    ax.errorbar(f['B'], np.array(f['fr'])*1e-9, 
                yerr=np.array(f['err'])*1e-9,
                marker='o',
                markersize='3',
                linestyle='', elinewidth = 0.7,
                color=color, label = label )

    ax.get_figure().tight_layout()

    return ax

def plot_fstheory(ax=None, fname='f_bz57.json', color='black'):
    """plot f_bz57.json f_stheory """
    with open( f'{export_dir}/'+fname ) as infile:
        f = json.load( infile )
    
    if ax is None: 
        fig, ax = plt.subplots()
    
    ax.plot(f['Btheory'], np.array(f['f_stheory']),
                linestyle='-', linewidth = 0.7,
                color=color, label = "Stheory" )


    upband = np.array(f['fup_stheory']) 
    lowband = np.array(f['flow_stheory'])

    ax.fill_between(f['Btheory'], upband, lowband,
                color=color, alpha=0.3, label = "uncertainty")

    ax.get_figure().tight_layout()

    return ax

def format_fplot( ax ):
    """Labelling and formatting the f plots"""

    ax.xaxis.set_major_formatter(major_formatter)
    ax.tick_params(axis='both', which='major', pad=1)
    ax.tick_params( direction='out' )
    tick0p5 = ax.xaxis.get_major_ticks()[1]
    tick0p5.label.set_position( ( 10, 0 ) )

    ax.set_xlim([0,0.85])
    ax.set_xlabel( '$\mathrm{B}$ (T)' , labelpad=-7, x=0.5 )
    ax.set_xticks( ( 0, 0.2, 0.4, 0.6, 0.8 ) )
    ax.set_xticklabels( ['0', None, None, None, '0.8'], fontsize=11 )

    ax.set_ylabel( '$\mathrm{f} (GHz)$', labelpad=-10, y=0.45 )
    ax.set_ylim([4.6, 5.65])
    ax.set_yticks( ( 4.6, 4.8, 5.0, 5.2, 5.4, 5.6 ) )
    ax.set_yticklabels( ('4.6', None, None, None, None, '5.6'), fontsize=11 )

    ax.legend([r"f($\mathrm{B_y}$)", r"f($\mathrm{B_x}$)"])
    ax.legend(["Stheory", "_Hidden",r"$f_r(\mathrm{B_y})$", r"$f_r(\mathrm{B_x})$"], 
                bbox_to_anchor=(1, 1.), loc="upper right",
                fontsize=10, handlelength=1, fancybox=False )
    
    return ax
    
fig, ax = plt.subplots(figsize=[3.5,3.2])
plot_f(ax, fname='f_by57.json', color='black', label=r"f($\mathrm{B_y}$)")
plot_f(ax, fname='f_bz57.json', color='green', label=r"f($\mathrm{B_x}$)")
plot_fstheory( ax, fname='f_bz57.json', color='red')

format_fplot(ax)
fig.subplots_adjust(top=0.96,
                    bottom=0.10,
                    left=0.11,
                    right=0.96
)

fig.savefig(f'{export_dir}/sup57.pdf', format='pdf', dpi=1200)
fig.show()

# %%
