#%% Plot angle sweep s58 from angle_58.json
import numpy as np, json
import matplotlib.pyplot as plt
from pathlib import Path
from figures import figpath

export_dir = figpath / 'supfigs'

def _load_file( f ) :
    ''' load file and convert to nparrays
    returns: dictionary with data in it
    '''
    with open( f ) as json_file :
        data_str = json.load( json_file )
    data = dict()
    for k in data_str : # convert strings to ints
        data[ float( k ) ] = data_str[ k ]

    for sweep in data.values() : # convert to np arrays
        for k, l in sweep.items() :
            sweep[ k ] = np.array( l )
    return data

def _get_min_f( data ) :
    ''' return minimum frequency in an angle sweep
    '''
    get_smallest_f = lambda sweep : min( sweep['f'] )
    smallest_f = min( map( get_smallest_f, data.values() ) )
    return smallest_f

def plot_angle_experiment( fig = None, smallest_f=None ):
    ''' load and plot angle sweep
    smallest_f: smallest frequency to plot
    '''
    f = f'{export_dir}/angle_58.json'
    data = _load_file( f )
    if smallest_f is None:
        smallest_f = _get_min_f( data )/1e9

    if fig is None:
        fig = plt.figure()
    ax = fig.add_axes( [0.20, 0.1, 0.9, 0.8],
                projection='polar' )

    for b, sweeps in data.items() :
        ax.errorbar( sweeps['angle']*np.pi/180, sweeps['f']/1e9, 
                yerr= sweeps['f_err' ]/1e9, linestyle='',
                linewidth=0.5, marker='o', 
                markersize=1, label=f'$|B|$ = {b}T' )

    fontdict = {'fontsize': 10,
                'fontweight': 1,
                'verticalalignment': 'center',
                'horizontalalignment': 'center'}
    ax.set_rlim( smallest_f )
    ax.set_yticklabels( [] )
    ax.set_xticks( np.pi/180. * np.linspace(0,360,8,endpoint=False))
    ax.set_xticklabels(['0$\degree$', None, '90$\degree$', None, 
                        '180$\degree$', None, '270$\degree$', None],
                        fontdict = fontdict
                        )
    ax.tick_params( pad = 2 )

    ax.legend( fontsize=11, loc='upper left', 
                bbox_to_anchor=(0,1),
                bbox_transform=fig.transFigure )


fig = plt.figure( figsize=[4.4, 3.2])
plot_angle_experiment( fig=fig, smallest_f=None )

fig.savefig(f'{export_dir}/sup58angle.pdf', format='pdf', dpi=1200)
fig.show()
# %%