#%%
import matplotlib.pyplot as plt, matplotlib as mpl
import numpy as np, json

from figures import figpath

# filled region settings
fill_args = { 'color' : '#C95011', 'alpha' : 0.15, 'linewidth' : 0 }
dsample = 2 # how much to downsample experimental data

def load_symasym( file = 'fsymasym.json') :
    """ load fsym/fasym dataset
    Returns:
        data as a dictionary, with entries as nparrays
    """
    fpath = figpath/'fig3'
    with open( f'{fpath}/{file}' ) as json_file :
        data = json.load( json_file )
    for k in data :
        data[k] = np.array( data[k] )
    return data

data = load_symasym()

def plot_fsymasym( ax_sym  ) :
    """ make fsym ax_sym
    """
    frange = ( 5.20, 5.45 ) # y-range for fsym plot

    marker_options = { 'color' : '#234F81',
                'linestyle' : '',
                'markersize' : 1,
                'fmt' : 'o',
                'mfc' : 'None',
                'elinewidth' : 0.5 }
    
    ax_sym.errorbar( data['phi_exp'][::dsample], 
            data['fsym'][::dsample]/1e9, 
            yerr=data['dfsym'][::dsample]/1e9, 
            **marker_options )
    ax_sym.plot( data['phi_theory'], 
            data['fsym_p91_theory'], 
            color='#234F81', linewidth=0.7 )
    ax_sym.plot( data['phi_theory'], 
        data['fsym_inf_theory'], 
        color='#234F81', linewidth=0.7 )
    ax_sym.hlines( 5.44906155, 0, 360 )
    ax_sym.set_yticks( frange )
    # fill in unstable squares
    ax_sym.set_ylim( 5.15, 5.48 )

    fill_lims = ( (45, 180-45), (180+45, 360-45) )
    for a in ( ax_sym, ) :
        for xmin, xmax in fill_lims :
            a.axvspan(xmin, xmax, **fill_args)
        a.set_xlabel( '$\\theta$ (deg)', labelpad=-10 )
        a.set_xticks( (0, 360 ) )
    ax_sym.set_ylabel( '$f_{r,\mathrm{sym}}$ (GHz)', 
            labelpad=-8, y=0.52 )

def plot_chi( ax ) :
    """ plot \chi^2(B_y*)
    """
    ax.plot( data['bs_chi'], data['chis']/1e4,
            marker='.', markersize=2,
            color='black' )
    ax.set_xlabel( '$B_y^*$', labelpad=-8, x=0.65 )
    ax.set_ylabel( '$\chi^2$ ($10^4$)', labelpad=-5 )
    
    ax.set_ylim( (-0.3,9) )
    ax.set_yticks( (0,4,8) )
    ax.set_yticklabels( (0,None,8) )

    ax.set_xscale('log')
    ax.set_xlim(right=10)
  
fig, ax = plt.subplots( 1,2, figsize=(4,1.8) )
plot_fsymasym( ax[0] )
plot_chi( ax[1] )
# subplot labels and legend
coords = { 'a' : ( 0.05, 0.94 ),
           'b' : ( 0.515, 0.94 )
        }
for lbl, xy in coords.items() :
    fig.text( *xy, lbl, 
            fontweight = 'bold',
            fontsize = 10 )
fig.tight_layout( pad=1, w_pad=0 )
# %%
fig.savefig( "gy.pdf" )
